
/* 
-------------------------------------------------------------------------------
-- Title      : 1-Wire  Master
-- Project    : 
-------------------------------------------------------------------------------
-- File       : onewire_master.v
-- Author     : Rick Ballantyne rickb@Xilinx.com
-- Company    : Xilinx, Inc.
-- Created    : 2001/03/15
-- Last Update: 2001/04/12
-- Copyright  : (c) Xilinx Inc, 2001
-------------------------------------------------------------------------------
-- Uses       : sr1.v, sr2.v, bit_reg.v, crc_reg.v, jcnt1.v, jcnt2.v, parallel_sn_data.v 
-------------------------------------------------------------------------------
-- Used by    : onewire_iface.v
-------------------------------------------------------------------------------
-- Description: This is the master module to drive the Serial Number Device.
--
--              When communicate with the Serial Number Device, this module
--              works as the master, while the Serial Number Device works as
--              slave. For more information about the Serial Number Device,
--              please refer to the datasheet at:
--              http://www.dalsemi.com/datasheets/pdfs/2401.pdf
--
--              This module has been verified to work with Dallas DS2401 
--              Silicon Serial Number Device and DS2430A EEPROM.
--
--              The function provided by this master module include:
--              (1) Send "Reset Pulse" to the Serial Number Device to reset it
--              (2) Detect "Presence Pulse" from the Serial Number Device
--              (3) Control data flow on the bidirectional one-wire bus which
--                  connects the Serial Number Device and this master module
--                  through one-wire.
--              (4) Read in the 8 bytes of data from the Serial Number Device
--                  which include the family code (x01), the serial number
--                  (6 bytes), and the CRC value (1 byte)
--              (5) Output the data to the data bus (data) as individual
--                  bytes (total 8 bytes) with a data enable signal (data_valid)
--                  as the strobe signal.
--                  The data bytes follow the sequence:
--                      1. Family Code:    (e.g. 0x01 for DS2401 device)
--                      2. Serial Number (Byte 0) 
--                      3. Serial Number (Byte 1) 
--                      4. Serial Number (Byte 2) 
--                      5. Serial Number (Byte 3) 
--                      6. Serial Number (Byte 4) 
--                      7. Serial Number (Byte 5) 
--                      8. CRC Value 
--              (6) Output the six Serial number bytes in parallel (sn_data)
--              (7) (optional) Calculate CRC and match it with the CRC value
--                  received from the device.
--              (8) Assert CRC OK if [a] all the bytes has been received and
--                  sent out to the data bus, and [b] CRC values are
--                  matched ([b] is optional).
--
--
--              This module needs an 1MHz (1us period) clock input.
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author          Description
-- 2001/03/15  1.0      Ballantyne      Conversion from VHDL
-- 2001/03/21  1.0      Ballantyne      added registers to the IO buffer data_out and ctrl
-- 2001/03/21  1.0      Ballantyne      added registers to data_valid crc_ok
-- 2001/04/12  1.0      Huang           detect presence of pull-up together with presence pulse    
-------------------------------------------------------------------------------

*/
module ONEWIRE_MASTER (clk, reset, dq, data_out, data_valid, crc_ok, sn_data, sys_clk); 
input           clk;
input           sys_clk;
input           reset;
inout           dq;
output  [7:0]   data_out;
output          data_valid;
output          crc_ok;
output [47:0]   sn_data;


wire clk_1MHz;
wire reset;


reg [2:0] PRESENT_STATE;
reg [2:0] NEXT_STATE;

// define the state machine state names
parameter [2:0]
INIT        = 3'b000,   // Reset/Initialization state
TX_RST_PLS  = 3'b001,   // Transmit Reset Pulse state
RX_PRE_PLS  = 3'b011,   // Receive Presence Detect state
TX_RD_CMD   = 3'b111,   // Transmit ROM READ command state
RX_DATA     = 3'b110,   // Recieve Serial Number Data state
IDLE        = 3'b100;   // Idle state


/*
  -- constant to issue Read ROM Command for DS2401 Serial Number Device
  -- which is either 0x33h (for both DS2401 and DS2430A)
  -- or 0x0Fh (for DS2401 only).
*/
parameter [7:0] ReadROMCmd = 8'b00110011;


// -- command data bit to transmit
reg         tx_cmd_bit;

//  -- internal generated clock (50KHz)
wire        clk_50KHz;
  
//  -- time slot identification signals
wire        ts_60_to_80us;
wire        ts_0_to_10us;
wire        ts_0_to_1us;
wire        ts_14_to_15us;
  

//  -- signals for shift register 1 (SR1) BIT position counter
reg             sr1_reset;
reg             sr1_en;
wire    [7:0]   sr1_q;

//  -- signals for shift register 2 (SR2) BYTE transfer counter
reg             sr2_reset;
reg             sr2_en;
wire    [7:0]   sr2_q;

//  -- signals for Johnson counter 1(JC1)
reg             jc1_reset;
wire    [1:0]   jc1_q;
    
//  -- signals for Johnson counter 2(JC2)
wire    [9:0]   jc2_q;

// -- signals for the bidirectional data I/O buffer and data path
wire        data_from_one_wire_bus;     // data from one-wire bus
reg         data_to_one_wire_bus;       // data to one-wire bus
reg         read_one_wire_dq;           // if 0 then dq <= data_to_one_wire_bus (write to the bus)
                                        // if 1 then data_from_one_wire_bus <= dq (read from the bus)
reg         data_from_one_wire_bus_pp;  // data of presence pulse 
                                        // it'll be 0 if presence pulse is detected.
reg         d_ctrl;                     // registered version of read_one_wire_dq
reg         d_out;                      // registered version of data_to_one_wire_bus
  
//  -- signals for bit register (BitReg)
wire    [7:0]   bitreg_en;      // enable signal to load one bit of data into the register
wire    [7:0]   bitreg_data;
  
//  -- several data valid signals
reg     databit_valid;  // databit_valid signal generated from sr1
                            // (which identifies states), it's
                            // 1us pulse. It indicates
                            // the valid data received from the 
                            // Serial Number Device, excludes
                            // the Presence Pulse.
reg     databyte_valid; // valid signal for receiving a byte of
                            // the number data from the Serial Number Device.
                            // Includes: family code, serial number
                            // and crc value. It's 1 us pulse.
  
//  -- signals for CRC check circuit
wire            crcreg_en;      // enable one bit data loaded into the CRC Register.
wire    [7:0]   crc_value;      // The calculated CRC value from the CRC register

// -- data outputs
wire    [47:0]  sn_data;        // parallel serial number data
wire    [7:0]   data_out;       // data output one byte at a time

// -- status flag outputs
reg     crc_ok	= 0;         // crc ok output registered version of crc 
reg     data_valid;     // registered version of databyte_valid
reg     crc;            // crc match flag

always @ (negedge clk_1MHz) begin
    data_valid <= databyte_valid;
    end

// -- some signals for wiring
wire        high;
wire        low;
assign high = 1'b1;
assign low  = 1'b0;
assign data_out[7:0] = bitreg_data[7:0];

/*
  -------------------------------------------------------------------
  -- Clock generation
  -------------------------------------------------------------------
*/
assign clk_50KHz = !jc2_q[9]; // use the msb of JC2 to generate
                               // 50KHz slow clock
                               // use "not" here to generate rising
                               // edge at proper position

assign clk_1MHz = clk;

/*                               
  -------------------------------------------------------------------
  -- Several time slot identification signals
  -------------------------------------------------------------------
  -- Suppose the beginning of each state is time 0.
  -- Use combination of JC1 and JC2, we can id any time slot during
  -- each state as small as 1 us.
*/

assign  ts_60_to_80us  = ( jc1_q[1] & !jc1_q[0]);
assign  ts_0_to_10us   = (!jc1_q[1] & !jc1_q[0] & !jc2_q[9]);
assign  ts_0_to_1us    = (!jc1_q[1] & !jc1_q[0] & !jc2_q[9] & !jc2_q[0]);
assign  ts_14_to_15us  = (!jc1_q[1] & !jc1_q[0] &  jc2_q[4] & !jc2_q[3]);  



/*
  -------------------------------------------------------------------
  -- ROM Command data bit to transmit (write) to the one wire bus
  -- Use SR1 to pick up each bit out of the Command data byte.
  -------------------------------------------------------------------
*/
always @ (sr1_q[7:0]) begin
    case (sr1_q)
    8'b00000001:begin
        tx_cmd_bit = ReadROMCmd[0];
        end
    8'b00000010:begin
        tx_cmd_bit = ReadROMCmd[1];
        end
    8'b00000100:begin
        tx_cmd_bit = ReadROMCmd[2];
        end
    8'b00001000:begin
        tx_cmd_bit = ReadROMCmd[3];
        end
    8'b00010000:begin
        tx_cmd_bit = ReadROMCmd[4];
        end
    8'b00100000:begin
        tx_cmd_bit = ReadROMCmd[5];
        end
    8'b01000000:begin
        tx_cmd_bit = ReadROMCmd[6];
        end
    8'b10000000:begin
        tx_cmd_bit = ReadROMCmd[7];
        end
default:begin
        tx_cmd_bit = 1'b0;
        end
        endcase
    end
 
/*
  -------------------------------------------------------------------
  -- Bidirectional iobuffer to control the direction of data flow on
  -- the one-wire bus
  -------------------------------------------------------------------  
*/

IOBUF IOBUF(
.O(data_from_one_wire_bus), 
.IO(dq), 
.I(d_out), 
.T(d_ctrl)
); 

always @ (negedge sys_clk) begin
    d_ctrl <= read_one_wire_dq;
    d_out <= data_to_one_wire_bus;
    end

/*
  -------------------------------------------------------------------
  -- Shift Register 1 
  -- Used to count 8 bits in a byte of data
  -------------------------------------------------------------------
*/

SR1 SR1 (
.clk(clk_50KHz),
.reset(sr1_reset),
.en(sr1_en),
.q(sr1_q[7:0])
);

/*              
  -------------------------------------------------------------------
  -- Shift Register 2  
  -- Used to count 8 bytes stored in the Serial Number Device
  -------------------------------------------------------------------
*/

SR2 SR2 (
.clk(clk_50KHz),
.reset(sr2_reset),
.en(sr2_en),
.q(sr2_q)
);

/*
  -------------------------------------------------------------------
  -- Johnson Counter 1  
  -- This Johnson counter is used to deal with the time slots.
  -- It chops one state into small time slots.Each is 20 us long,
  -- total 4 slots. 
  -- The reason to use Johnson Counter is to save register bits.
  -- n bits of Johnson counter can count for 2n values.
  -- for the shift register, n bits can only count for n values
  -- In addition, Johnson counter is fast since only NOT gates
  -- are used in this counter.
  -- It's driven by the slow clock (20us) in this system.
  -- Counter transitions 00 -> 01 -> 11 -> 10 -> 00
  -------------------------------------------------------------------
*/

JCNT1 JCNT1 (
.clk(clk_50KHz),
.reset(jc1_reset),
.en(high),
.q(jc1_q[1:0])
);

/*
  -------------------------------------------------------------------
  -- Johnson Counter 2 
  -- (1) Use this counter to generate 20 us slow clock (jc2_q[9]).
  -- (2) It is also used to divide a period of time into time slots.
  --     It counts for small time slot which
  --     is 1 us wide, add up to total 20 slots. 
  -- It should be synchronized with JCount1. 
  
  -------------------------------------------------------------------
*/

JCNT2 JCNT2 (
.clk(clk_1MHz),
.reset(reset),
.en(high),
.q(jc2_q[9:0])
);

/*
  -------------------------------------------------------------------
  -- Bit Register
  -- It accumulates 8 bits of data accoring to the strobes of
  -- bitreg_en, and output a byte of data.
  -------------------------------------------------------------------
*/
/*    
   -- This is the enable signal for the BitReg. When a bit of data
   -- is ready, the BitReg is enabled to load this bit of data
   -- at corresponding data bit. For example: bitreg_en = "00001000"
   -- means the data bit will be stored into the register bit 3.
   -- This enable strobe is asserted only one clock period of 1 us.
   -- It's generated from the output of SR1, which is used to
   -- count for 8 bits in a byte of data. It is also controlled by
   -- the numberbits_valid, which indicates a valid data bit received
   -- from the Serial Number Device.
*/
assign  bitreg_en[0] = databit_valid ? sr1_q[0] : low;
assign  bitreg_en[1] = databit_valid ? sr1_q[1] : low;
assign  bitreg_en[2] = databit_valid ? sr1_q[2] : low;
assign  bitreg_en[3] = databit_valid ? sr1_q[3] : low;
assign  bitreg_en[4] = databit_valid ? sr1_q[4] : low;
assign  bitreg_en[5] = databit_valid ? sr1_q[5] : low;
assign  bitreg_en[6] = databit_valid ? sr1_q[6] : low;
assign  bitreg_en[7] = databit_valid ? sr1_q[7] : low;

BITREG BITREG(
.clk(clk_1MHz), 
.reset(reset), 
.din(data_from_one_wire_bus),
.en(bitreg_en[7:0]), 
.dout(bitreg_data[7:0])
);



 // create a parallel output for the serial number this will be a valid
 // serial number if the crc_ok signal is high

PARALLEL_SN_DATA PARALLEL_SN_DATA(
.clk(clk_50KHz), 
.reset(reset), 
.data(bitreg_data[7:0]), 
.enable(sr2_q[6:1]), 
.sn_data(sn_data[47:0])
);


 /*
   ------------------------------------------------------------------------
   -- Check CRC and Generate crc_ok signal
   -- Use CRCReg to calculate CRC value
   -- Latch crcok result when the last state is entered (IDLE state)
   --
   -- Note: Use the parameter CheckCRC to turn on or off this CRC check circuit.
   -- If CheckCRC is false, this circuit will be removed to save
   -- register resources. And crc_ok output will be asserted high as
   -- long as all the data bytes are received. So it won't reflect the CRC
   -- checking result.
   ------------------------------------------------------------------------

   --
   -- If we employ CRC check circuit,
   --
   */
`ifdef CheckCRC

/*           
       -- This enable signal is generated for each data bit when we
       -- receive the crc data from the one-wire device. It's 1us pulse.
       -- use sr2_q(7) here to exclude the situation when we receive
       -- the crc value from the one-wire device.
*/
   assign crcreg_en = databit_valid & !sr2_q[7];

// instantiate the CRC generation module
CRCReg CRCReg(
.clk(clk_1MHz),
.reset(reset),
.en(crcreg_en),
.din(data_from_one_wire_bus),
.q(crc_value[7:0])
);


/*
      -- Assert crc_ok signal when the last byte of data (crc_value) is
      -- received and it matches the caculated crc value from the CRCReg
*/
    always @ (reset or databyte_valid or sr2_q[7] or crc_value[7:0] or bitreg_data[7:0]) begin
        if (reset) begin
            crc <= 1'b0;
            end
        else if (databyte_valid & sr2_q[7]) begin
                if (bitreg_data[7:0] == crc_value[7:0]) begin   // last data byte and crc matches
                    crc <= 1'b1;
                    end
            else begin
                crc <= 1'b0;                            // crc does not match
            end 
        end
        else begin
            crc <= 1'b0;                                // not the last data byte
            end
        end

`else

/*
   --
   -- If we do not use CRC check circuit,
   --
*/


//     -- Assert crc_ok when the all the data bytes have been received
    always @ (reset or databyte_valid or sr2_q[7]) begin
        if (reset) begin
            crc <= 1'b0;
            end
        else if (databyte_valid & sr2_q[7]) begin
            crc <= 1'b1;                        // last data byte
            end
        else begin
            crc <= 1'b0;
            end
        end
`endif


// register the crc ok flag 
    always @ (posedge crc or posedge reset) begin
    if (reset) begin
        crc_ok <= 1'b0; 
        end 
    else begin
        crc_ok <= 1'b1;
        end
    end

/*
   -----------------------------------------------------------------------------
   -- The FSM register
   -----------------------------------------------------------------------------
*/
// synthesis translate_off
initial begin
        PRESENT_STATE <= INIT;
        end
// synthesis translate_on

always @ (posedge clk_50KHz or posedge reset) begin
    if (reset) begin
        PRESENT_STATE <= INIT;
        end
    else begin
        PRESENT_STATE <= NEXT_STATE;
        end
    end
/*
  ------------------------------------------------------------------------
   -- State Mux 
   -- Combinational Logic for the state machine.
   --
   -- Any action in this state mux is synchronized with the 20 us clock
   -- and a few of them are synchronized with the 1us clock.
   --
   -- The transition of the state will take effect on next
   -- rising edge of the clock. 
   ------------------------------------------------------------------------
*/ 
  
always @ (PRESENT_STATE or ts_0_to_1us or ts_0_to_10us or ts_14_to_15us or ts_60_to_80us or sr2_q[6] or  sr2_q[0]
            or sr1_q[7] or data_from_one_wire_bus_pp  or data_from_one_wire_bus or tx_cmd_bit or crc_ok ) begin

case (PRESENT_STATE)
/*
                             ---------------------------------------
                             -- Reset/Initialization state
                             ---------------------------------------
                             -- The one-wire bus will be pulled up,
                             -- so that next state we can send a
                             -- Reset Pulse (active low) to the bus.
                             ---------------------------------------
*/
INIT:begin
    sr1_reset               = 1'b1;
    sr1_en                  = 1'b0;
    sr2_reset               = 1'b1;
    sr2_en                  = 1'b0;
    read_one_wire_dq        = 1'b0;     // drive the one-wire bus high
    data_to_one_wire_bus    = 1'b1;
    databyte_valid          = 1'b0;
    databit_valid           = 1'b0;
    jc1_reset               = 1'b1;

    NEXT_STATE = TX_RST_PLS;
end

/*
                                ---------------------------------------
                               -- Transmit Reset Pulse state     
                               ---------------------------------------
                               -- In this state, the one-wire bus will
                               -- be pulled down (Tx "Reset Pulse") for
                               -- 480 us to reset the one-wire
                               -- device connected to the bus.
                               --
                               -- It enables FSM to move to next state
                               -- at 480 us. The transition of the state
                               -- will happend at 500 us.
                               --
                               -- Use JC1 and SR2 here to count for
                               -- longer time duration (0 ~ 480 us):
                               -----------------------------------------
                               -- Time    JC1 SR2  SR2    SR2
                               -- elapse      En   Rst
                               --  (us)                 msb     lsb
                               -----------------------------------------
                               --    0    "00" '0' '0'  "00000001"
                               --    20   "01" '0' '0'  "00000001"
                               --    40   "11" '0' '0'  "00000001" 
                               --    60   "10" '1' '0'  "00000001"
                               --    80   "00" '0' '0'  "00000010"
                               --   100   "01" '0' '0'  "00000010"
                               --   120   "11" '0' '0'  "00000010"
                               --   140   "10" '1' '0'  "00000010"
                               --   160   "00" '0' '0'  "00000100"
                               --   180   "01" '0' '0'  "00000100"
                               --   ...   ...  ... ...      ...
                               --   ...   ...  ... ...      ...
                               --   240   "00" '0' '0'  "00001000"
                               --   ...   ...  ... ...      ...
                               --   320   "00" '0' '0'  "00010000"
                               --   ...   ...  ... ...      ...
                               --   400   "00" '0' '0'  "00100000"
                               --   ...   ...  ... ...      ...
                               --   480   "00" '0' '1'  "01000000" 
                               --   500   "01" '0' '0'  "00000001"
                               ----------------------------------------
*/

TX_RST_PLS:begin
    sr1_reset               = 1'b1;
    sr1_en                  = 1'b0;

    sr2_en                  = ts_60_to_80us;// enable sr2 to count every 80us

    databyte_valid          = 1'b0;
    databit_valid           = 1'b0;


    if (sr2_q[6]) begin                     // 480 us has passed.
        read_one_wire_dq        = 1'b1;
        data_to_one_wire_bus    = 1'b1;
        jc1_reset               = 1'b1; // reset the 20 us counter  
        sr2_reset               = 1'b1;

        NEXT_STATE = RX_PRE_PLS; 
        end
    else begin                              // 0 ~ 480 us
        read_one_wire_dq        = 1'b0;     // write the one-wire bus with "0"
        data_to_one_wire_bus    = 1'b0;     // for 480 us (one-wire RESET)
        jc1_reset               = 1'b0;
        sr2_reset               = 1'b0;     // use sr2 to create a 480 us counter

        NEXT_STATE = TX_RST_PLS;
        end
end
                 
/*
                               ---------------------------------------
                               -- Detect Presence Pulse state     
                               ---------------------------------------
                               -- In this state, data on the one-wire
                               -- bus is sampled for the presence of a slave.
                               -- The data will be latched at 0~80 us.
                               -- Then it waits till total 500us has
                               -- has passed, and moves to next state
                               -- or goes back to INIT state according 
                               -- to the presence of the "Presence
                               -- Pulse"
                               --
                               -- Use JC1 and SR2 here to count for
                               -- longer time duration (0 ~ 480 us):
                               --
                               -- Note:"Presence Pulse" indicates
                               -- a Serial Number Device is on the bus
                               -- and it's ready to operate.
                               ----------------------------------------
*/
        
RX_PRE_PLS:begin
    sr1_reset               = 1'b1;
    sr1_en                  = 1'b0;
    sr2_reset               = 1'b0;         // use sr2 to create a 480 us counter
    sr2_en              = ts_60_to_80us;    // enable sr2 to count every 80us
    read_one_wire_dq        = 1'b1;
    data_to_one_wire_bus    = 1'b0;
    databyte_valid          = 1'b0;
    databit_valid           = 1'b0;


    if (sr2_q[6]) begin                                 // wait for 480us to pass
        if (data_from_one_wire_bus_pp == 1'b0 & data_from_one_wire_bus == 1'b1) begin   // slave is present and pull-up is present
            jc1_reset   = 1'b1;
        NEXT_STATE = TX_RD_CMD;

            end
        else begin                                      // no slave present try again
            jc1_reset   = 1'b0;
            NEXT_STATE = INIT;
            end
        end
    else begin                                          // 0 ~ 480 us
        jc1_reset   = 1'b0;
        NEXT_STATE = RX_PRE_PLS;
        end
end


/*
                               ---------------------------------------
                               -- Transmit ROM Function Command state
                               ---------------------------------------
                               -- In this state, the one-wire bus is
                               -- pulled down during first 10 us.
                               --
                               -- Then according to each bit of data
                               -- in the ROM Command (0x0F for Read ROM),
                               -- we write data to the Serial Number
                               -- Device:
                               -- (1) if we need to write '1' to the serial
                               -- number device, it will release
                               -- the one-wire bus to allow the
                               -- pull-up resistor to pull the wire to '1'.
                               -- (2) if we need to write '0' to
                               -- the device, we output '0' directly
                               -- to the bus. This process happens from
                               -- 10 us to 60 us.
                               -- 
                               -- After 60us, it releases the bus allowing
                               -- the one-wire bus to be pulled back to
                               -- high, and enable SR1 to shift to 
                               -- next bit.
                               --  
                               -- After another 20us, the transition of SR1
                               -- will take place. The process will repeat
                               -- to transmit another bit in the ROM Command,
                               -- till all 8 bits in the ROM Command have
                               -- been sent out. 
                               --
                               -- After 8 bits of data has been sent out,
                               -- it moves to next state
                               -----------------------------------------
*/
TX_RD_CMD:begin
    sr1_reset               = 1'b0; // use sr1 to count the 8 bits in the read ROM command 

    sr2_reset               = 1'b1;
    sr2_en                  = 1'b0;

    databyte_valid          = 1'b0;
    databit_valid           = 1'b0;


    if (ts_0_to_10us) begin         // pull down one_wire bus
        read_one_wire_dq        = 1'b0;
        data_to_one_wire_bus    = 1'b0;
        sr1_en                  = 1'b0;
        jc1_reset               = 1'b0;

        NEXT_STATE = TX_RD_CMD;
        end

    else if (ts_60_to_80us) begin   // release the bus
        read_one_wire_dq        = 1'b1;
        data_to_one_wire_bus    = 1'b1;
        sr1_en                  = 1'b1; // bit has been sent enable the next bit

        if (sr1_q[7]) begin         // all command bits have been sent
            jc1_reset   = 1'b1;

            NEXT_STATE = RX_DATA;
            end
        else begin                  // sent more bits
            jc1_reset   = 1'b0;

            NEXT_STATE = TX_RD_CMD;
            end
        end

    else begin                      // write the command bit from 10 us to 60 us
        read_one_wire_dq        = tx_cmd_bit;
        data_to_one_wire_bus    = tx_cmd_bit;
        sr1_en                  = 1'b0;
        jc1_reset               = 1'b0;

        NEXT_STATE = TX_RD_CMD;     // send more read command bits
        end
end


/*
                               ---------------------------------------
                               -- Receive Serial Number Data state
                               ---------------------------------------
                               -- In this state, the onewire bus is
                               -- pulled down during first 1 us, this
                               -- is the initialization of the Rx of one
                               -- bit . Then it release the bus by changing
                               -- back to read mode.
                               --
                               -- From 13us to 15 us, it samples the 
                               -- data on the one-wire bus, and assert
                               -- databit_valid signal. 
                               --
                               -- After 15us, it releases the bus allowing
                               -- the one-wire bus to be pulled back to
                               -- high.
                               --
                               -- At 60us, it enables SR1 to shift to
                               -- next bit. After 80us, one bit has
                               -- been read. Then it repeats
                               -- the process to receive other 7
                               -- bits in one byte.
                               --
                               -- After 8 bits for one byte of data have
                               -- been received, the SR2 is used to
                               -- count for total 8 bytes of serial
                               -- number data.
                               --
                               -- After 8 bytes of data has been received,
                               -- it moves to next state
                               -----------------------------------------
*/


RX_DATA:begin
    sr1_reset               = 1'b0; // start to use sr1 to count 8 bits.
                                    // sr1 is configured to wrap around
                                    // when it reach the end. So it's
                                    // not necessary to reset it.

    sr2_reset               = 1'b0; // start to use sr2 to count the
                                    // 8 bytes coming from the one-wire device

    read_one_wire_dq        = 1'b1;
    data_to_one_wire_bus    = 1'b1;


    if (ts_0_to_1us) begin      // pull down one_wire bus
        read_one_wire_dq        = 1'b0;
        data_to_one_wire_bus    = 1'b0;
        databyte_valid          = 1'b0;
        databit_valid           = 1'b0;
        sr1_en                  = 1'b0;
        sr2_en                  = 1'b0;
        jc1_reset               = 1'b0;

        NEXT_STATE = RX_DATA;
        end

    else if (ts_60_to_80us) begin
        read_one_wire_dq        = 1'b1;
        data_to_one_wire_bus    = 1'b1;
        databyte_valid          = sr1_q[7]; // enable data output to external
                                            // world when all 8 bits are read.

        databit_valid           = 1'b0;
        sr1_en                  = 1'b1;     // one bit has been read, enable the next bit
        sr2_en                  = sr1_q[7]; // byte has been read, enable the next word

            if (sr2_q[7] & databyte_valid) begin
                jc1_reset   = 1'b1;

                NEXT_STATE = IDLE;          // all 8 bytes have been received
                end
            else begin
                jc1_reset   = 1'b1;
                NEXT_STATE = RX_DATA;       // get more data
                end
            end

    else begin                              // 1-60us
        read_one_wire_dq        = 1'b1;     // release the bus
        data_to_one_wire_bus    = 1'b1;
        databyte_valid          = 1'b0;
        databit_valid           = ts_14_to_15us;
        sr1_en                  = 1'b0;
        sr2_en                  = 1'b0;
        jc1_reset               = 1'b0;

        NEXT_STATE = RX_DATA;       // continue to assemble the data bytes
        end
end         

/*
                               ---------------------------------------
                               -- IDLE state
                               ---------------------------------------
                               -- The onewire bus will be released to
                               -- read mode; the data bus will keep the
                               -- last byte (CRC value); crcok will
                               -- be valid as a latch signal.
                               --
                               -- Once enter IDLE state, it stays here
                               -- unless getting a system reset signal.
                               ---------------------------------------
*/

IDLE:begin
    sr1_reset               = 1'b1;
    sr1_en                  = 1'b0;
    sr2_reset               = 1'b1;
    sr2_en                  = 1'b0;
    read_one_wire_dq        = 1'b1;
    data_to_one_wire_bus    = 1'b1;
    databyte_valid          = 1'b0;
    databit_valid           = 1'b0;
    jc1_reset               = 1'b0;

    if (crc_ok) begin
        NEXT_STATE = IDLE;
        end
    else begin
        NEXT_STATE = INIT;
        end
    end


default:begin
    sr1_reset               = 1'b1;
    sr1_en                  = 1'b0;
    sr2_reset               = 1'b1;
    sr2_en                  = 1'b0;
    read_one_wire_dq        = 1'b1;
    data_to_one_wire_bus    = 1'b1;
    databyte_valid          = 1'b0;
    databit_valid           = 1'b0;
    jc1_reset               = 1'b0;

    NEXT_STATE = INIT;
    end

endcase
end


// capture the presence data if "0" then the slave is present otherwise the pullup is read

always @ (posedge ts_60_to_80us or posedge reset) begin
    if (reset) begin
        data_from_one_wire_bus_pp = 1'b1;                       // default to NOT present
        end
    else if (PRESENT_STATE == RX_PRE_PLS & sr2_q[0]) begin      // first 60us slot 
            data_from_one_wire_bus_pp = data_from_one_wire_bus; // capture the presence bit
            end
    end


endmodule
module IOBUF(O, IO, I, T); // synthesis syn_black_box
output O;
inout IO;
input I;
input T;
endmodule
